@@ -17,9 +17,10 @@ our @EXPORT_OK = qw(
 );
 our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 
-sub _CERTIFICATE_HEADER_SIZE { return 32; }
+sub _CERTIFICATE_HEADER_SIZE { return 40; }
+sub _MAX_PUBLIC_KEY_SIZE     { return 65_536; }
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 sub pin_sha256 {
     my ($path) = @_;
@@ -27,28 +28,99 @@ sub pin_sha256 {
       or Carp::croak("Failed to open $path for reading:$EXTENDED_OS_ERROR");
     read $handle, my $file_header, _CERTIFICATE_HEADER_SIZE()
       or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
-    if ( $file_header !~ /^[-]{5}BEGIN[ ]CERTIFICATE[-]{5}\r?\nMII/smx ) {
-        Carp::croak("$path is not a PEM encoded SSL Certificate");
+    my $pem_encoded_public_key_string;
+    if ( $file_header =~
+        /^[-]{5}BEGIN[ ](?:X[.]?509[ ]|TRUSTED[ ])?CERTIFICATE[-]{5}/smx )
+    {
+        $pem_encoded_public_key_string =
+          _process_certificate( $handle, $file_header, $path );
     }
-    my $x509 = Crypt::OpenSSL::X509->new_from_file($path);
-    my $string;
-    if ( $x509->key_alg_name() eq 'rsaEncryption' ) {
-        my $pubkey = Crypt::OpenSSL::RSA->new_public_key( $x509->pubkey() );
-        $string = $pubkey->get_public_key_x509_string();
+    elsif ( $file_header =~ /^[-]{5}BEGIN[ ](?:RSA[ ])?PUBLIC[ ]KEY[-]{5}/smx )
+    {
+        $pem_encoded_public_key_string =
+          _process_public_key( $handle, $file_header, $path );
     }
     else {
-        $string = $x509->pubkey();
+        Carp::croak("$path is not a PEM encoded SSL Certificate");
     }
-    $string =~ s/^[-]+BEGIN[^\n]+\n//smx;
-    $string =~ s/^[-]+END[^\n]+\n//smx;
-    my $der    = MIME::Base64::decode($string);
+
+    $pem_encoded_public_key_string =~
+      s/^[-]{5}BEGIN[ ]PUBLIC[ ]KEY[-]{5}\r?\n//smx;
+    $pem_encoded_public_key_string =~
+      s/^[-]{5}END[ ]PUBLIC[ ]KEY[-]{5}\r?\n//smx;
+    my $der_encoded_public_key_string =
+      MIME::Base64::decode($pem_encoded_public_key_string);
     my $digest = Digest->new('SHA-256');
-    $digest->add($der);
+    $digest->add($der_encoded_public_key_string);
     my $base64 = MIME::Base64::encode_base64( $digest->digest() );
     chomp $base64;
     return $base64;
 }
 
+sub _process_certificate {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    if ( $file_header =~ /^[-]{5}BEGIN[ ]CERTIFICATE[-]{5}/smx ) {
+        my $x509 = Crypt::OpenSSL::X509->new_from_file($path);
+        $pem_encoded_public_key_string =
+          _get_pem_encoded_public_key_string($x509);
+    }
+    elsif ( $file_header =~
+        /^[-]{5}BEGIN[ ](?:X[.]?509|TRUSTED)[ ]CERTIFICATE[-]{5}/smx )
+    {
+        seek $handle, 0, Fcntl::SEEK_SET()
+          or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+        defined read $handle, my $pem_encoded_certificate_string,
+          _MAX_PUBLIC_KEY_SIZE()
+          or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+        $pem_encoded_certificate_string =~
+s/^([-]{5}BEGIN[ ])(?:X[.]?509|TRUSTED)[ ](CERTIFICATE[-]{5})/$1$2/smx;
+        $pem_encoded_certificate_string =~
+          s/^([-]{5}END[ ])(?:X[.]?509|TRUSTED)[ ](CERTIFICATE[-]{5})/$1$2/smx;
+        my $x509 = Crypt::OpenSSL::X509->new_from_string(
+            $pem_encoded_certificate_string);
+        $pem_encoded_public_key_string =
+          _get_pem_encoded_public_key_string($x509);
+    }
+    return $pem_encoded_public_key_string;
+}
+
+sub _process_public_key {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    if ( $file_header =~ /^[-]{5}BEGIN[ ]PUBLIC[ ]KEY[-]{5}/smx ) {
+        seek $handle, 0, Fcntl::SEEK_SET()
+          or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+        defined read $handle, $pem_encoded_public_key_string,
+          _MAX_PUBLIC_KEY_SIZE()
+          or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+    }
+    elsif ( $file_header =~ /^[-]{5}BEGIN[ ]RSA[ ]PUBLIC[ ]KEY[-]{5}/smx ) {
+        seek $handle, 0, Fcntl::SEEK_SET()
+          or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+        defined read $handle, my $pem_encoded_rsa_public_key_string,
+          _MAX_PUBLIC_KEY_SIZE()
+          or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+        my $pubkey = Crypt::OpenSSL::RSA->new_public_key(
+            $pem_encoded_rsa_public_key_string);
+        $pem_encoded_public_key_string = $pubkey->get_public_key_x509_string();
+    }
+    return $pem_encoded_public_key_string;
+}
+
+sub _get_pem_encoded_public_key_string {
+    my ($x509) = @_;
+    my $pem_encoded_public_key_string;
+    if ( $x509->key_alg_name() eq 'rsaEncryption' ) {
+        my $pubkey = Crypt::OpenSSL::RSA->new_public_key( $x509->pubkey() );
+        $pem_encoded_public_key_string = $pubkey->get_public_key_x509_string();
+    }
+    else {
+        $pem_encoded_public_key_string = $x509->pubkey();
+    }
+    return $pem_encoded_public_key_string;
+}
+
 1;    # End of HTTP::PublicKeyPins
 __END__
 
@@ -58,7 +130,7 @@ HTTP::PublicKeyPins - Generate RFC 7469 HTTP Public Key Pin (HPKP) header values
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =head1 SYNOPSIS
 
@@ -85,7 +157,7 @@ This module allows the calculation of RFC 7469 HTTP Public Key Pin header values
 
 =head2 pin_sha256
 
-This function accepts the path to a certificate.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value.
+This function accepts the path to a PEM encoded certificate.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM format).  Patches for other formats are encouraged.
 
 =head1 SUBROUTINES/METHODS
 
@@ -178,7 +250,6 @@ L<http://search.cpan.org/dist/HTTP-PublicKeyPins/>
 Copyright 2015 David Dick.
 
 This module is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself. See L<perlartistic|perlartistic>.  This program is
-distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
-without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.
+the same terms as Perl itself. This program is distributed in the hope that 
+it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
+warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
